/***************************************************************************
***
***    Copyright 2008  Hon Hai Precision Ind. Co. Ltd.
***    All Rights Reserved.
***    No portions of this material shall be reproduced in any form without the
***    written permission of Hon Hai Precision Ind. Co. Ltd.
***
***    All information contained in this document is Hon Hai Precision Ind.  
***    Co. Ltd. company private, proprietary, and trade secret property and 
***    are protected by international intellectual property laws and treaties.
***
****************************************************************************/

/*******************************************************************************
 *      Extended LED control implementation, also can be used as normally
 *      GPIO control by using ioctl EXT_LED_SET
 ******************************************************************************/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>
#include <asm/time.h>
#include <typedefs.h>
#include <bcmutils.h>
#include <siutils.h>
#include <bcmdevs.h>

#ifdef GPIO_EXT_CTRL

#include <ext_led.h>
#include <wps_led.h>

//#define _DEBUG

#define EXT_LED_CLK  1 

#ifdef CONFIG_DEVFS_FS
static int ext_led_major;
devfs_handle_t ext_leddev_handle;
#endif

#if (defined INCLUDE_USB_LED)
extern int usb_led_state;
#endif

extern int wlan_n_state; //2.4G radio
extern int wlan_g_state; //5G radio

extern unsigned int ext_led_status;

extern int ext_output_value(int led_status);
extern int gpio_led_on_off(int gpio, int value);

static int
ext_led_open(struct inode *inode, struct file * file)
{
    MOD_INC_USE_COUNT;
    return 0;
}

static int
ext_led_release(struct inode *inode, struct file * file)
{
    MOD_DEC_USE_COUNT;
    return 0;
}

static int
ext_led_ioctl(struct inode *inode, struct file *file, unsigned int cmd, int arg)
{
    struct gpio_exchange gpioioc;

    switch (cmd)
    {
        case EXT_CLOCK_ON:
            //ext_clock_onoff=1;
            break;

        case EXT_CLOCK_OFF:
            //ext_clock_onoff=0;
            break;

        case EXT_LED_SET:
            copy_from_user(&gpioioc, (struct gpio_exchange *)arg, sizeof(struct gpio_exchange));
            gpio_led_on_off(gpioioc.gpio_pin, gpioioc.gpio_value);
            break;

        case EXT_LED_GET:
            copy_to_user((unsigned int *)arg, &ext_led_status, sizeof(unsigned int));
            break;

        case EXT_SET_ALL:
            /* all ON but power green */
            wlan_n_state = 0;
            wlan_g_state = 0;
            #if (defined INCLUDE_USB_LED)
            usb_led_state = 0;
            #endif
            ext_led_status = EXT_LED_STATUS_ALL_ON;
            gpio_led_on_off(GPIO_LED_PWR_AMBER, 0);
            break;

        case EXT_CLR_SET:
            /* all OFF but power amber */
            wlan_n_state = 0;
            wlan_g_state = 0;
            #if (defined INCLUDE_USB_LED)
            usb_led_state = 0;
            #endif
            ext_led_status = EXT_LED_STATUS_ALL_OFF;
            gpio_led_on_off(GPIO_LED_PWR_AMBER, 1);
            break;

        case EXT_SET_MULTI_LED:
            ext_led_status = (unsigned int)arg;
            ext_output_value(ext_led_status);
            break;

        /* Foxconn added start pling 02/26/2010 */
        /* Add USB LED support. */
#if (defined INCLUDE_USB_LED)
        case EXT_USB_LED_STATE_ON:
            usb_led_state = 1;
            break;
        
        case EXT_USB_LED_STATE_OFF:
            usb_led_state = 0;
            break;
#endif
        /* Foxconn added end pling 02/26/2010 */

        /* 2.4G radio on*/
        case EXT_WLAN_N_RADIO_ON:
            wlan_n_state = 1;
            gpio_led_on_off(WLAN_N_LED_GPIO, 1);
            if (wlan_g_state == 0)
                gpio_led_on_off(WLAN_LED_GPIO, 1);
            break;
         
        /* 2.4G radio off*/
        case EXT_WLAN_N_RADIO_OFF:
            wlan_n_state = 0;
            gpio_led_on_off(WLAN_N_LED_GPIO, 0);
            if (wlan_g_state == 0)
                gpio_led_on_off(WLAN_LED_GPIO, 0);
            break;
            
        /* 5G radio on*/
        case EXT_WLAN_G_RADIO_ON:
            wlan_g_state = 1;
            gpio_led_on_off(WLAN_G_LED_GPIO, 1);
            if (wlan_n_state == 0)
                gpio_led_on_off(WLAN_LED_GPIO, 1);
            break;
            
        /* 5G radio on*/
        case EXT_WLAN_G_RADIO_OFF:
            wlan_g_state = 0;
            gpio_led_on_off(WLAN_G_LED_GPIO, 0);
            if (wlan_n_state == 0)
                gpio_led_on_off(WLAN_LED_GPIO, 0);
            break;

        case DOME_N_RADIO_ON:
            break;
         
        /* 2.4G radio off*/
        case DOME_N_RADIO_OFF:
            break;
            
        /* 5G radio on*/    
        case DOME_G_RADIO_ON:
            break;  
            
        /* 5G radio on*/    
        case DOME_G_RADIO_OFF:
            break;
            
        default:
            break;
    }

    return 0;

}
static struct file_operations ext_led_fops = {
#ifndef CONFIG_DEVFS_FS
    .owner      = THIS_MODULE,
    .open       = ext_led_open,
    .release    = ext_led_release,
    .ioctl      = ext_led_ioctl,
#else
    owner:      THIS_MODULE,
    open:       ext_led_open,
    release:    ext_led_release,
    ioctl:      ext_led_ioctl,
#endif
    };

static int __init
ext_led_init(void)
{
#ifndef CONFIG_DEVFS_FS
    int ret_val;
    ret_val = register_chrdev(EXT_LED_MAJOR_NUM, "ext_led", &ext_led_fops);
    /*
     * Negative values signify an error
     */
    if (ret_val < 0)
    {
        printf("%s failed with %d\n","Sorry, registering the character device ext_led", ret_val);
        return ret_val;
    }
#else
    if ((ext_led_major = devfs_register_chrdev(EXT_LED_MAJOR_NUM, "ext_led", &ext_led_fops)) < 0)
        return ext_led_major;

    ext_leddev_handle = devfs_register(NULL, "ext_led", DEVFS_FL_DEFAULT,
                                    ext_led_major, 0, S_IFCHR | S_IRUGO | S_IWUGO,
                                    &ext_led_fops, NULL);
#endif
    return 0;
}

static void __exit
ext_led_exit(void)
{
#ifndef CONFIG_DEVFS_FS
    int ret_val;
    ret_val = unregister_chrdev(EXT_LED_MAJOR_NUM, "ext_led");
#else
    if (ext_leddev_handle != NULL)
        devfs_unregister(ext_leddev_handle);
    ext_leddev_handle = NULL;
    devfs_unregister_chrdev(ext_led_major, "ext_led");
#endif
}

module_init(ext_led_init);
module_exit(ext_led_exit);

#endif /* GPIO_EXT_CTRL */
