/***************************************************************************
***
***    Copyright 2008  Hon Hai Precision Ind. Co. Ltd.
***    All Rights Reserved.
***    No portions of this material shall be reproduced in any form without the
***    written permission of Hon Hai Precision Ind. Co. Ltd.
***
***    All information contained in this document is Hon Hai Precision Ind.  
***    Co. Ltd. company private, proprietary, and trade secret property and 
***    are protected by international intellectual property laws and treaties.
***
****************************************************************************/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/uaccess.h>

#include <typedefs.h>
#include <bcmutils.h>
#include <siutils.h>
#include <bcmdevs.h>

#include "wps_led.h"

//#define _DEBUG

#ifdef CONFIG_DEVFS_FS
static int wps_led_major;
devfs_handle_t wps_leddev_handle;
#endif /* CONFIG_DEVFS_FS */

extern int wps_led_pattern;
extern int wps_led_state;
extern int is_wl_secu_mode; /* foxconn added, zacker, 09/17/2009, @wps_led */

static int
wps_led_open(struct inode *inode, struct file * file)
{
    MOD_INC_USE_COUNT;
    return 0;
}

static int
wps_led_release(struct inode *inode, struct file * file)
{
    MOD_DEC_USE_COUNT;
    return 0;
}

static int
wps_led_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
    /* foxconn added start, zacker, 09/17/2009, @wps_led */
    if (arg)
        is_wl_secu_mode = 1;
    else
        is_wl_secu_mode = 0;
    /* foxconn added end, zacker, 09/17/2009, @wps_led */

    switch (cmd)
    {
        case WPS_LED_BLINK_NORMAL:
            wps_led_state = 1;
#ifdef _DEBUG
            printk("%s: blink normal\n", __FUNCTION__);
#endif
            break;

        case WPS_LED_BLINK_QUICK:
            wps_led_state = 2;
#ifdef _DEBUG
            printk("%s: blink WPS\n", __FUNCTION__);
#endif
            break;

        case WPS_LED_BLINK_OFF:
            /* foxconn added start, zacker, 09/17/2009, @wps_led */
            /* wps_led_state will change to 0 automatically after
             * blinking a few seconds if it's 2 currently
             */
            if (wps_led_state != 2)
            /* foxconn added end, zacker, 09/17/2009, @wps_led */
                wps_led_state = 0;
#ifdef _DEBUG
            printk("%s: blink OFF\n", __FUNCTION__);
#endif
            break;

        case WPS_LED_CHANGE_GREEN:
            break;

        case WPS_LED_CHANGE_AMBER:
            break;

        case WPS_LED_BLINK_QUICK2:
            wps_led_state = 3;
            break;

        default:
            break;
    }

    return 0;
}

static struct file_operations wps_led_fops = {
#ifndef CONFIG_DEVFS_FS
    .owner      = THIS_MODULE,
    .open       = wps_led_open,
    .release    = wps_led_release,
    .ioctl      = wps_led_ioctl,
#else /* CONFIG_DEVFS_FS */
    owner:      THIS_MODULE,
    open:       wps_led_open,
    release:    wps_led_release,
    ioctl:      wps_led_ioctl,
#endif /* CONFIG_DEVFS_FS */
    };

static int __init
wps_led_init(void)
{
#ifndef CONFIG_DEVFS_FS
    int ret_val;
    /*
    * Register the character device (atleast try)
    */
    ret_val = register_chrdev(WPS_LED_MAJOR_NUM, "wps_led", &wps_led_fops);
    /*
    * Negative values signify an error
    */
    if (ret_val < 0) 
    {
        printf("%s failed with %d\n","Sorry, registering the character device wps_led", ret_val);
        return ret_val;
    } 
#else /* CONFIG_DEVFS_FS */
    if ((wps_led_major = devfs_register_chrdev(WPS_LED_MAJOR_NUM, "wps_led", &wps_led_fops)) < 0)
        return wps_led_major;

    wps_leddev_handle = devfs_register(NULL, "wps_led", DEVFS_FL_DEFAULT,
                                    wps_led_major, 0, S_IFCHR | S_IRUGO | S_IWUGO,
                                    &wps_led_fops, NULL);
#endif /* CONFIG_DEVFS_FS */
    return 0;
}

static void __exit
wps_led_exit(void)
{
#ifndef CONFIG_DEVFS_FS
    int ret_val;
    ret_val = unregister_chrdev(WPS_LED_MAJOR_NUM, "wps_led");
#else /* CONFIG_DEVFS_FS */
    if (wps_leddev_handle != NULL)
        devfs_unregister(wps_leddev_handle);
    wps_leddev_handle = NULL;
    devfs_unregister_chrdev(wps_led_major, "wps_led");
#endif /* CONFIG_DEVFS_FS */
}

module_init(wps_led_init);
module_exit(wps_led_exit);
