/*
 *	Generic parts
 *	Linux ethernet bridge
 *
 *	Authors:
 *	Lennert Buytenhek		<buytenh@gnu.org>
 *
 *	$Id: br.c,v 1.1.1.1 2007-08-03 18:53:50 Exp $
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/init.h>
#include <linux/llc.h>
#include <net/llc.h>

#include "br_private.h"

int (*br_should_route_hook) (struct sk_buff **pskb) = NULL;

static struct llc_sap *br_stp_sap;

static int __init br_init(void)
{
	int err;

	br_stp_sap = llc_sap_open(LLC_SAP_BSPAN, br_stp_rcv);
	if (!br_stp_sap) {
		printk(KERN_ERR "bridge: can't register sap for STP\n");
		return -EADDRINUSE;
	}

	err = br_fdb_init();
	if (err)
		goto err_out1;

	err = br_netfilter_init();
	if (err)
		goto err_out1;

	err = register_netdevice_notifier(&br_device_notifier);
	if (err)
		goto err_out2;

	err = br_netlink_init();
	if (err)
		goto err_out3;

	brioctl_set(br_ioctl_deviceless_stub);
	br_handle_frame_hook = br_handle_frame;

	br_fdb_get_hook = br_fdb_get;
	br_fdb_put_hook = br_fdb_put;

	/* Foxconn added start pling 03/13/2007 */
#define nvram_safe_get(name) (nvram_get(name) ? : "")
	char *qos_en = nvram_safe_get("qos_enable");
	char *wla_rp = nvram_safe_get("wla_repeater");
	if(!strcmp(qos_en, "1") && strcmp(wla_rp, "1")) {
		extern int qos_enable;
		qos_enable = 1;
	}
	/* Foxconn added end pling 03/13/2007 */

	return 0;
err_out3:
	unregister_netdevice_notifier(&br_device_notifier);
err_out2:
	br_netfilter_fini();
err_out1:
	llc_sap_put(br_stp_sap);
	return err;
}

static void __exit br_deinit(void)
{
	rcu_assign_pointer(br_stp_sap->rcv_func, NULL);

	br_netlink_fini();
	br_netfilter_fini();
	unregister_netdevice_notifier(&br_device_notifier);
	brioctl_set(NULL);

	br_cleanup_bridges();

	synchronize_net();

	llc_sap_put(br_stp_sap);
	br_fdb_get_hook = NULL;
	br_fdb_put_hook = NULL;

	br_handle_frame_hook = NULL;
	br_fdb_fini();
}

EXPORT_SYMBOL(br_should_route_hook);
/* Foxconn move start, Max Ding, 03/03/2011 build bridge as kernel module to shrink kernel size */
/* Foxconn add start, Zz Shan@MutiSsidControl 03/13/2009*/
#ifdef MULTIPLE_SSID
extern void insert_msc_func_to_br(void *FUNC);
EXPORT_SYMBOL(insert_msc_func_to_br);
extern void remove_msc_func_from_br();
EXPORT_SYMBOL(remove_msc_func_from_br);

/*foxconn modified start, water, 01/07/10*/
//#include "../../../router/multissidcontrol/MultiSsidControl.h"
#include "../../../../../ap/acos/multissidcontrol/MultiSsidControl.h"
/*foxconn modified end, water, 01/07/10*/
extern T_MSsidCtlProfile *gProfile;
EXPORT_SYMBOL(gProfile);
extern int gProfilenum;
EXPORT_SYMBOL(gProfilenum);
/* Foxconn added start pling 10/06/2010 */
extern T_MSsidCtlProfile *gProfile_5g;
EXPORT_SYMBOL(gProfile_5g);
extern int gProfilenum_5g;
EXPORT_SYMBOL(gProfilenum_5g);
/* Foxconn added end pling 10/06/2010 */
#endif
/* Foxconn add end, Zz Shan 03/13/2009*/
/* Foxconn move end, Max Ding, 03/03/2011 */

module_init(br_init)
module_exit(br_deinit)
MODULE_LICENSE("GPL");
MODULE_VERSION(BR_VERSION);
