/*
 * Miscellaneous services
 *
 * Copyright (C) 2010, Broadcom Corporation. All Rights Reserved.
 * 
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * $Id: services.c 281156 2011-09-01 08:30:55Z kenlo $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <errno.h>

#include <bcmnvram.h>
#include <netconf.h>
#include <shutils.h>
#include <rc.h>
#include <pmon.h>

#define assert(a)

#ifdef __CONFIG_NAT__
static char
*make_var(char *prefix, int index, char *name)
{
	static char buf[100];

	assert(prefix);
	assert(name);

	if (index)
		snprintf(buf, sizeof(buf), "%s%d%s", prefix, index, name);
	else
		snprintf(buf, sizeof(buf), "%s%s", prefix, name);
	return buf;
}
int
start_dhcpd(void)
{
	FILE *fp;
	char name[100];
	char word[32], *next;
	char dhcp_conf_file[128];
	char dhcp_lease_file[128];
	char dhcp_ifnames[128] = "";
	int index;
	char tmp[20];
	int i = 0;
	char *lan_ifname = NULL;

	if (nvram_match("router_disable", "1"))
		return 0;

	/* Setup individual dhcp severs for the bridge and the every unbridged interface */
	for (i = 0; i < MAX_NO_BRIDGE; i++) {
		if (!i)
			snprintf(tmp, sizeof(tmp), "lan_ifname");
		else
			snprintf(tmp, sizeof(tmp), "lan%x_ifname", i);

		lan_ifname = nvram_safe_get(tmp);

		if (!strcmp(lan_ifname, ""))
			continue;

		snprintf(dhcp_ifnames, sizeof(dhcp_ifnames), "%s %s", dhcp_ifnames, lan_ifname);
	}

	index = 0;
	foreach(word, dhcp_ifnames, next) {

		if (strstr(word, "br0"))
			index = 0;
		else
			index = 1;

		/* Skip interface if no valid config block is found */
		if (index < 0) continue;

		if (nvram_invmatch(make_var("lan", index, "_proto"), "dhcp"))
			continue;

		dprintf("%s %s %s %s\n",
			nvram_safe_get(make_var("lan", index, "_ifname")),
			nvram_safe_get(make_var("dhcp", index, "_start")),
			nvram_safe_get(make_var("dhcp", index, "_end")),
			nvram_safe_get(make_var("lan", index, "_lease")));

		/* Touch leases file */
		sprintf(dhcp_lease_file, "/tmp/udhcpd%d.leases", index);
		if (!(fp = fopen(dhcp_lease_file, "a"))) {
			perror(dhcp_lease_file);
			return errno;
		}
		fclose(fp);

		/* Write configuration file based on current information */
		sprintf(dhcp_conf_file, "/tmp/udhcpd%d.conf", index);
		if (!(fp = fopen(dhcp_conf_file, "w"))) {
			perror(dhcp_conf_file);
			return errno;
		}
		fprintf(fp, "pidfile /var/run/udhcpd%d.pid\n", index);
		fprintf(fp, "start %s\n", nvram_safe_get(make_var("dhcp", index, "_start")));
		fprintf(fp, "end %s\n", nvram_safe_get(make_var("dhcp", index, "_end")));
		fprintf(fp, "interface %s\n", word);
		fprintf(fp, "remaining yes\n");
		fprintf(fp, "lease_file /tmp/udhcpd%d.leases\n", index);
		fprintf(fp, "option subnet %s\n",
			nvram_safe_get(make_var("lan", index, "_netmask")));
		fprintf(fp, "option router %s\n",
			nvram_safe_get(make_var("lan", index, "_ipaddr")));
		fprintf(fp, "option dns %s\n", nvram_safe_get(make_var("lan", index, "_ipaddr")));
		fprintf(fp, "option lease %s\n", nvram_safe_get(make_var("lan", index, "_lease")));
		snprintf(name, sizeof(name), "%s_wins",
			nvram_safe_get(make_var("dhcp", index, "_wins")));
		if (nvram_invmatch(name, ""))
			fprintf(fp, "option wins %s\n", nvram_get(name));
		snprintf(name, sizeof(name), "%s_domain",
			nvram_safe_get(make_var("dhcp", index, "_domain")));
		if (nvram_invmatch(name, ""))
			fprintf(fp, "option domain %s\n", nvram_get(name));
		fclose(fp);

		eval("udhcpd", dhcp_conf_file);
		index++;
	}
	dprintf("done\n");
	return 0;
}

int
stop_dhcpd(void)
{
	char sigusr1[] = "-XX";
	int ret;

/*
* Process udhcpd handles two signals - SIGTERM and SIGUSR1
*
*  - SIGUSR1 saves all leases in /tmp/udhcpd.leases
*  - SIGTERM causes the process to be killed
*
* The SIGUSR1+SIGTERM behavior is what we like so that all current client
* leases will be honorred when the dhcpd restarts and all clients can extend
* their leases and continue their current IP addresses. Otherwise clients
* would get NAK'd when they try to extend/rebind their leases and they
* would have to release current IP and to request a new one which causes
* a no-IP gap in between.
*/
	sprintf(sigusr1, "-%d", SIGUSR1);
	eval("killall", sigusr1, "udhcpd");
	ret = eval("killall", "udhcpd");

	dprintf("done\n");
	return ret;
}

int
start_dns(void)
{
	int ret;
	FILE *fp;
	char dns_ifnames[64] = "";
	char tmp[20];
	int i = 0;
	char *lan_ifname = NULL;
	char dns_cmd[255];

	if (nvram_match("router_disable", "1"))
		return 0;

	/* Create resolv.conf with empty nameserver list */
	if (!(fp = fopen("/tmp/resolv.conf", "w"))) {
		perror("/tmp/resolv.conf");
		return errno;
	}
	fclose(fp);

	/* Setup interface list for the dns relay */
	for (i = 0; i < MAX_NO_BRIDGE; i++) {
		if (!i)
			snprintf(tmp, sizeof(tmp), "lan_ifname");
		else
			snprintf(tmp, sizeof(tmp), "lan%x_ifname", i);

		lan_ifname = nvram_safe_get(tmp);

		if (!strcmp(lan_ifname, ""))
			continue;

		snprintf(dns_ifnames, sizeof(dns_ifnames), "%s -i %s", dns_ifnames, lan_ifname);
	}

	/* Start the dns relay */
	sprintf(dns_cmd, "/usr/sbin/dnsmasq -h -n %s -r /tmp/resolv.conf&", dns_ifnames);
	ret = system(dns_cmd);

	dprintf("done\n");
	return ret;
}

int
stop_dns(void)
{
	int ret = eval("killall", "dnsmasq");

	/* Remove resolv.conf */
	unlink("/tmp/resolv.conf");

	dprintf("done\n");
	return ret;
}
#endif	/* __CONFIG_NAT__ */

/*
*/

int
start_httpd(void)
{
	int ret;

	chdir("/www");
	ret = eval("httpd");
	chdir("/");

	dprintf("done\n");
	return ret;
}

int
stop_httpd(void)
{
	int ret = eval("killall", "httpd");

	dprintf("done\n");
	return ret;
}

#ifdef PLC
/* Run time check for if gigled must be started based on nvram keys */
static int
target_uses_gigled(void)
{
	return nvram_match("wl0_plc", "1") || nvram_match("wl1_plc", "1");
}

static int
start_gigled(void)
{
	int ret;

	chdir("/tmp");
	ret = eval("/usr/sbin/gigled");
	chdir("/");

	dprintf("done\n");
	return ret;
}

static int
stop_gigled(void)
{
	int ret = eval("killall", "gigled");

	dprintf("done\n");
	return ret;
}
#endif

#ifdef __CONFIG_NAT__
#ifdef __CONFIG_UPNP__
int
start_upnp(void)
{
	char *wan_ifname;
	int ret;
	char var[100], prefix[] = "wanXXXXXXXXXX_";

	if (!nvram_invmatch("upnp_enable", "0"))
		return 0;

	ret = eval("killall", "-SIGUSR1", "upnp");
	if (ret != 0) {
	    snprintf(prefix, sizeof(prefix), "wan%d_", wan_primary_ifunit());
	    wan_ifname = nvram_match(strcat_r(prefix, "proto", var), "pppoe") ?\
				nvram_safe_get(strcat_r(prefix, "pppoe_ifname", var)) :\
				nvram_safe_get(strcat_r(prefix, "ifname", var));

		ret = eval("upnp", "-D", "-W", wan_ifname);

	}
	dprintf("done\n");
	return ret;
}

int
stop_upnp(void)
{
	int ret = 0;

	if (nvram_match("upnp_enable", "0"))
	    ret = eval("killall", "upnp");

	dprintf("done\n");
	return ret;
}
#endif /* __CONFIG_UPNP__ */

#ifdef	__CONFIG_IGD__
int
start_igd(void)
{
	int ret = 0;

	if (nvram_match("upnp_enable", "1")) {
		ret = eval("igd", "-D");
	}

	return ret;
}

int
stop_igd(void)
{
	int ret = 0;

	ret = eval("killall", "igd");

	return ret;
}
#endif	/* __CONFIG_IGD__ */
#endif	/* __CONFIG_NAT__ */



int
start_nas(void)
{
	int ret = eval("nas");

#ifdef __CONFIG_MEDIA_IPTV__

	pid_t pid;
	pid_t prev_pid = -1;
	int found = 0;

	/* Iterate a few times to allow nas to finish daemonizing itself. */
	do {
		sleep(1);
		if ((pid = get_pid_by_name("nas")) <= 0)
			break;
		found = (pid == prev_pid) ? found + 1 : 1;
		prev_pid = pid;
	} while (found < 3);

	if (found == 3)
		pmon_register(pid, &start_nas);

#endif /* __CONFIG_MEDIA_IPTV__ */

	return ret;
}

int
stop_nas(void)
{
	int ret;

#ifdef __CONFIG_MEDIA_IPTV__

	pid_t pid;

	if ((pid = pmon_get_pid(&start_nas)) > 0) 
		pmon_unregister(pid);

#endif /* __CONFIG_MEDIA_IPTV__ */

	ret = eval("killall", "nas");

	return ret;
}

#ifdef __CONFIG_WAPI__
int
start_wapid(void)
{
	int ret = eval("wapid");

	return ret;
}

int
stop_wapid(void)
{
	int ret = eval("killall", "wapid");

	return ret;
}
#endif /* __CONFIG_WAPI__ */

#ifdef __CONFIG_WAPI_IAS__
int
start_ias(void)
{
	char *ias_argv[] = {"ias", "-F", "/etc/AS.conf", "-D", NULL};
	pid_t pid;

	if (nvram_match("as_mode", "enabled")) {
		_eval(ias_argv, NULL, 0, &pid);
	}

	return 0;
}

int
stop_ias(void)
{
	int ret = 0;

	if (!nvram_match("as_mode", "enabled")) {
		/* Need add signal handler in as */
		ret = eval("killall", "ias");
	}

	return ret;
}
#endif /* __CONFIG_WAPI_IAS__ */

int
start_ntpc(void)
{
	char *servers = nvram_safe_get("ntp_server");

	if (strlen(servers)) {
		char *nas_argv[] = {"ntpclient", "-h", servers, "-i", "3", "-l", "-s", NULL};
		pid_t pid;

		_eval(nas_argv, NULL, 0, &pid);
	}

	dprintf("done\n");
	return 0;
}

int
stop_ntpc(void)
{
	int ret = eval("killall", "ntpclient");

	dprintf("done\n");
	return ret;
}

int
start_telnet(void)
{
	char tmp[20];
	int i = 0;
	int ret = 0;
	char *lan_ifname = NULL;

	for (i = 0; i < MAX_NO_BRIDGE; i++) {
		if (!i)
			snprintf(tmp, sizeof(tmp), "lan_ifname");
		else
			snprintf(tmp, sizeof(tmp), "lan%x_ifname", i);

		lan_ifname = nvram_safe_get(tmp);

		if (!strcmp(lan_ifname, ""))
			continue;

		ret = eval("utelnetd", "-d", "-i", lan_ifname);
	}

	return ret;
}

int
stop_telnet(void)
{
	int ret;
	ret = eval("killall", "utelnetd");
	return ret;
}


int
start_wps(void)
{
#ifdef __CONFIG_WPS__
	char *wps_argv[] = {"/bin/wps_monitor", NULL};
	pid_t pid;
	if ( nvram_match("wps_status", "2") && !nvram_match("wps_start", "pbc") )
	{
		return 0;
	}

	nvram_set("wps_status", "0");
	nvram_set("wps_method", "1");
	nvram_set("wps_config_command", "0");
	nvram_set("wps_proc_mac", "");
	nvram_set("wps_proc_status", "0");

	if (nvram_match("wps_restart", "1")) {
		nvram_set("wps_restart", "0");
	}
	else {
		nvram_set("wps_restart", "0");
		nvram_set("wps_proc_status", "0");
	}

	nvram_set("wps_sta_pin", "00000000");
	nvram_set("wps_currentband", "");

   	eval("killall","wps_monitor");
   	eval("killall","wps_ap");
   	eval("killall","wps_enr");
	_eval(wps_argv, NULL, 0, &pid);

#else
	/* if we don't support WPS, make sure we unset any remaining wl_wps_mode */
	nvram_unset("wl_wps_mode");
#endif /* __CONFIG_WPS__ */
	return 0;

}

int
stop_wps(void)
{
	int ret = 0;

#ifdef __CONFIG_WPS__
   	ret = eval("killall","wps_monitor");
   	ret = eval("rm", "-f", "/tmp/wps_monitor.pid"); 
   	ret = eval("killall","wps_ap");
#endif /* __CONFIG_WPS__ */

	return ret;
}

int
start_bcmupnp(void)
{
	int ret;

	if (!nvram_invmatch("upnp_enable", "0"))
	{
	    printf("upnp_enable is 0, needn't trigger bcmupnp.\n");
		return 0;
	}
		
    if ( nvram_match("wl0_wps_mode", "disabled") || nvram_match("wps_mode", "disabled") )
    {
        printf("WPS is disabled, needn't trigger bcmupnp.\n");
        return 0;
    }

    /* Restart upnp properly */
#if 0
	ret = eval("killall", "-SIGUSR1", "upnp");
	if (ret != 0) 
	{
		ret = eval("upnp", "-D", "-W", nvram_get("wan_ifname"));
	}
#endif
	eval("killall", "upnp");
	eval("upnp", "-D", "-W", nvram_get("wan_ifname"));
	dprintf("done\n");
	return ret;
}

int
stop_bcmupnp(void)
{
	int ret = 0;

	if (nvram_match("upnp_enable", "0"))
	    ret = eval("killall", "upnp");

	dprintf("done\n");
	return ret;
}

#ifdef __CONFIG_IGMP_PROXY__
void
start_igmp_proxy(void)
{
	/* Start IGMP Proxy in Router mode only */
	if (!nvram_match("router_disable", "1"))
		eval("igmp", nvram_get("wan_ifname"));
	return;
}

void
stop_igmp_proxy(void)
{
	eval("killall", "igmp");
	return;
}
#endif /* __CONFIG_IGMP_PROXY__ */

#ifdef __CONFIG_LLD2D__
int start_lltd(void)
{
	chdir("/usr/sbin");
	eval("lld2d", "br0");
	chdir("/");
	
	return 0;
}

int stop_lltd(void)
{
	int ret;
	
	ret = eval("killall", "lld2d");

	return ret;
}
#endif /* __CONFIG_LLD2D__ */

int
start_eapd(void)
{
	int ret = eval("/bin/eapd");

	return ret;
}

int
stop_eapd(void)
{
   	int ret = eval("killall","eapd");

   	return ret;
}

#if defined(BCM_DCS) || defined(EXT_ACS)
int
start_acsd(void)
{
	int ret = eval("/usr/sbin/acsd");

	return ret;
}

int
stop_acsd(void)
{
   	int ret = eval("killall","acsd");

   	return ret;
}
#endif /* BCM_DCS || EXT_ACS */


#if defined(PHYMON)
int
start_phymons(void)
{
	int ret = eval("/usr/sbin/phymons");

	return ret;
}

int
stop_phymons(void)
{
	int ret = eval("killall", "phymons");

	return ret;
}
#endif /*  PHYMON */

#ifdef __CONFIG_SAMBA__
void enable_gro(int interval)
{
#ifdef LINUX26
	char *argv[3] = {"echo", "", NULL};
	char lan_ifname[32], *lan_ifnames, *next;
	char path[64] = {0};
	char parm[32] = {0};

	/* enabled gso on vlan interface */
	lan_ifnames = nvram_safe_get("lan_ifnames");
	foreach(lan_ifname, lan_ifnames, next) {
		if (!strncmp(lan_ifname, "vlan", 4)) {
			sprintf(path, ">>/proc/net/vlan/%s", lan_ifname);
			sprintf(parm, "-gro %d", interval);	
			argv[1] = parm;
			_eval(argv, path, 0, NULL);
		}
	}
#endif
}

int
start_samba()
{
	char *argv[3] = {"echo", "", NULL};
	char *samba_mode;
	char *samba_passwd;

	enable_gro(2);

	samba_mode = nvram_safe_get("samba_mode");
	samba_passwd = nvram_safe_get("samba_passwd");

	/* Samba is disabled */
	if (strncmp(samba_mode, "1", 1) && strncmp(samba_mode, "2", 1))
		return 0;

	/* Create smb.conf */
	argv[1] = "[global]";
	_eval(argv, ">/tmp/samba/lib/smb.conf", 0, NULL);

	argv[1] = "workgroup = mygroup";
	_eval(argv, ">>/tmp/samba/lib/smb.conf", 0, NULL);

	if (!strncmp(samba_mode, "1", 1))
		argv[1] = "security = user";
	else
		argv[1] = "security = share";
	_eval(argv, ">>/tmp/samba/lib/smb.conf", 0, NULL);

#ifdef LINUX26
	argv[1] = "[media]";
#else
	argv[1] = "[mnt]";
#endif
	_eval(argv, ">>/tmp/samba/lib/smb.conf", 0, NULL);

#ifdef LINUX26
	argv[1] = "path = /media";
#else
	argv[1] = "path = /mnt";
#endif
	_eval(argv, ">>/tmp/samba/lib/smb.conf", 0, NULL);

	argv[1] = "writeable = yes";
	_eval(argv, ">>/tmp/samba/lib/smb.conf", 0, NULL);

	argv[1] = "browseable = yes";
	_eval(argv, ">>/tmp/samba/lib/smb.conf", 0, NULL);

	argv[1] = "guest ok = yes";
	_eval(argv, ">>/tmp/samba/lib/smb.conf", 0, NULL);

	/* Start smbd */
	eval("smbd", "-D");

	/* Set admin password */
	argv[1] = samba_passwd;
	_eval(argv, ">/tmp/spwd", 0, NULL);
	_eval(argv, ">>/tmp/spwd", 0, NULL);
	system("smbpasswd -a admin -s </tmp/spwd");

	return 0;
}

int
stop_samba()
{
	eval("killall", "smbd");
	eval("rm", "-r", "/tmp/samba/var/locks");
	eval("rm", "/tmp/samba/private/passdb.tdb");

	enable_gro(0);

	return 0;
}
#endif

#ifdef __CONFIG_DLNA_DMR__
int start_dlna_dmr()
{
	char *dlna_dmr_enable = nvram_safe_get("dlna_dmr_enable");

	if (strcmp(dlna_dmr_enable, "1") == 0) {
		cprintf("Start bcmmrenderer.\n");
		eval("sh", "-c", "bcmmrenderer&");
	}
}

int stop_dlna_dmr()
{
	cprintf("Stop bcmmrenderer.\n");
	eval("killall", "bcmmrenderer");
}
#endif


#ifdef __CONFIG_WL_ACI__
int start_wl_aci(void){
	int ret = 0;

	/*
	 * If the ACI daemon is enabled, start the ACI daemon.  If not
	 *  simply return
	 */
	if (!nvram_match("aci_daemon", "up"))
		return 0;

	ret = eval("wl_aci");
	return ret;
}

int
stop_wl_aci(void)
{
	int ret;

	ret = eval("killall", "wl_aci");
	return ret;
}
#endif /* __CONFIG_WL_ACI__ */

#ifdef RWL_SOCKET
int
start_server_socket(void)
{
	char *srv_argv[] = {"/usr/sbin/wl_server_socket", NULL};
	pid_t pid;

	_eval(srv_argv, NULL, 0, &pid);

	return 0;
}

int
stop_server_socket(void)
{
   	int ret = eval("killall","wl_server_socket");

   	return ret;
}
#endif /* RWL_SOCKET */

int
start_services(void)
{
/*
*/
	start_httpd();
#ifdef __CONFIG_NAT__
	start_dns();
	start_dhcpd();
#ifdef	__CONFIG_UPNP__	
	start_upnp();
#endif
#ifdef	__CONFIG_IGD__
	start_igd();
#endif
#endif	/* __CONFIG_NAT__ */
	start_eapd();
	start_nas();
#ifdef __CONFIG_WAPI_IAS__
	start_ias();
#endif /* __CONFIG_WAPI_IAS__ */
#ifdef __CONFIG_WAPI__
	start_wapid();
#endif /* __CONFIG_WAPI__ */
	start_wps();
#ifdef __CONFIG_UTELNETD__
	start_telnet();
#endif
#ifdef __CONFIG_IGMP_PROXY__
	start_igmp_proxy();
#endif /* __CONFIG_IGMP_PROXY__ */
#ifdef __CONFIG_WL_ACI__
	start_wl_aci();
#endif /* __CONFIG_WL_ACI__ */
#ifdef __CONFIG_LLD2D__
	start_lltd();
#endif /* __CONFIG_LLD2D__*/
#if defined(PHYMON)
	start_phymons();
#endif /* PHYMON */
#if defined(BCM_DCS) || defined(EXT_ACS)
	start_acsd();
#endif
#ifdef __CONFIG_SAMBA__
	start_samba();
#endif

#ifdef __CONFIG_DLNA_DMR__
	start_dlna_dmr();
#endif

#ifdef RWL_SOCKET
	start_server_socket();
#endif

#ifdef PLC
	if (target_uses_gigled())
		start_gigled();
#endif

	dprintf("done\n");
	return 0;
}

int
stop_services(void)
{
	stop_wps();
#ifdef __CONFIG_WAPI__
	stop_wapid();
#endif /* __CONFIG_WAPI__ */
#ifdef __CONFIG_WAPI_IAS__
	stop_ias();
#endif /* __CONFIG_WAPI_IAS__ */
	stop_nas();
	stop_eapd();
#ifdef __CONFIG_NAT__
#ifdef	__CONFIG_IGD__
	stop_igd();
#endif
#ifdef	__CONFIG_UPNP__
	stop_upnp();
#endif
	stop_dhcpd();
	stop_dns();
#endif	/* __CONFIG_NAT__ */
	stop_httpd();
#ifdef __CONFIG_UTELNETD__
	stop_telnet();
#endif
#ifdef __CONFIG_IGMP_PROXY__
	stop_igmp_proxy();
#endif /* __CONFIG_IGMP_PROXY__ */
#ifdef __CONFIG_WL_ACI__
        stop_wl_aci();
#endif /* __CONFIG_WL_ACI__ */
#ifdef __CONFIG_LLD2D__
	stop_lltd();
#endif 	/* __CONFIG_LLD2D__*/
/*
*/
#if defined(PHYMON)
	stop_phymons();
#endif /* PHYMON */
#if defined(BCM_DCS) || defined(EXT_ACS)
	stop_acsd();
#endif
#ifdef __CONFIG_SAMBA__
	stop_samba();
#endif
#ifdef __CONFIG_DLNA_DMR__
	stop_dlna_dmr();
#endif

#ifdef RWL_SOCKET
	stop_server_socket();
#endif

#ifdef PLC
	if (target_uses_gigled())
		stop_gigled();
#endif

	dprintf("done\n");
	return 0;
}
